@@ -124,35 +124,39 @@ def decompose(S, n_components=None, transformer=None, sort=False, fit=True, **kw
 
     Sort components by ascending peak frequency
 
-    >>> comps, acts = librosa.decompose.decompose(S, n_components=8,
+    >>> comps, acts = librosa.decompose.decompose(S, n_components=16,
     ...                                           sort=True)
 
 
     Or with sparse dictionary learning
 
     >>> import sklearn.decomposition
-    >>> T = sklearn.decomposition.MiniBatchDictionaryLearning(n_components=8)
-    >>> comps, acts = librosa.decompose.decompose(S, transformer=T, sort=True)
+    >>> T = sklearn.decomposition.MiniBatchDictionaryLearning(n_components=16)
+    >>> scomps, sacts = librosa.decompose.decompose(S, transformer=T, sort=True)
 
     >>> import matplotlib.pyplot as plt
     >>> plt.figure(figsize=(10,8))
     >>> plt.subplot(3, 1, 1)
-    >>> librosa.display.specshow(librosa.logamplitude(S**2,
-    ...                                               ref_power=np.max),
+    >>> librosa.display.specshow(librosa.amplitude_to_db(S,
+    ...                                                  ref=np.max),
     ...                          y_axis='log', x_axis='time')
     >>> plt.title('Input spectrogram')
     >>> plt.colorbar(format='%+2.0f dB')
     >>> plt.subplot(3, 2, 3)
-    >>> librosa.display.specshow(comps, y_axis='log')
+    >>> librosa.display.specshow(librosa.amplitude_to_db(comps,
+    ...                                                  ref=np.max),
+    ...                          y_axis='log')
+    >>> plt.colorbar(format='%+2.0f dB')
     >>> plt.title('Components')
     >>> plt.subplot(3, 2, 4)
     >>> librosa.display.specshow(acts, x_axis='time')
     >>> plt.ylabel('Components')
     >>> plt.title('Activations')
+    >>> plt.colorbar()
     >>> plt.subplot(3, 1, 3)
     >>> S_approx = comps.dot(acts)
-    >>> librosa.display.specshow(librosa.logamplitude(S_approx**2,
-    ...                                               ref_power=np.max),
+    >>> librosa.display.specshow(librosa.amplitude_to_db(S_approx,
+    ...                                                  ref=np.max),
     ...                          y_axis='log', x_axis='time')
     >>> plt.colorbar(format='%+2.0f dB')
     >>> plt.title('Reconstructed spectrogram')
@@ -183,7 +187,7 @@ def decompose(S, n_components=None, transformer=None, sort=False, fit=True, **kw
     return components, activations
 
 
-@cache
+@cache(level=30)
 def hpss(S, kernel_size=31, power=2.0, mask=False, margin=1.0):
     """Median-filtering harmonic percussive source separation (HPSS).
 
@@ -220,15 +224,18 @@ def hpss(S, kernel_size=31, power=2.0, mask=False, margin=1.0):
           of the percussive filter.
 
     power : float > 0 [scalar]
-        Exponent for the Wiener filter when constructing mask matrices.
-
-        Mask matrices are defined by
-        `mask_H = (r_H ** power) / (r_H ** power + r_P ** power)`
-        where `r_H` and `r_P` are the median-filter responses for
-        harmonic and percussive components.
+        Exponent for the Wiener filter when constructing soft mask matrices.
 
     mask : bool
-        Return the (binary) masking matrices instead of components
+        Return the masking matrices instead of components.
+
+        Masking matrices contain non-negative real values that
+        can be used to measure the assignment of energy from `S`
+        into harmonic or percussive components.
+
+        Components can be recovered by multiplying `S * mask_H`
+        or `S * mask_P`.
+
 
     margin : float or tuple (margin_harmonic, margin_percussive)
         margin size(s) for the masks (as described in [2]_)
@@ -251,6 +258,9 @@ def hpss(S, kernel_size=31, power=2.0, mask=False, margin=1.0):
     --------
     util.softmask
 
+    Notes
+    -----
+    This function caches at level 30.
 
     Examples
     --------
@@ -263,20 +273,20 @@ def hpss(S, kernel_size=31, power=2.0, mask=False, margin=1.0):
     >>> import matplotlib.pyplot as plt
     >>> plt.figure()
     >>> plt.subplot(3, 1, 1)
-    >>> librosa.display.specshow(librosa.logamplitude(np.abs(D)**2,
-    ...                                               ref_power=np.max),
+    >>> librosa.display.specshow(librosa.amplitude_to_db(D,
+    ...                                                  ref=np.max),
     ...                          y_axis='log')
     >>> plt.colorbar(format='%+2.0f dB')
     >>> plt.title('Full power spectrogram')
     >>> plt.subplot(3, 1, 2)
-    >>> librosa.display.specshow(librosa.logamplitude(np.abs(H)**2,
-    ...                          ref_power=np.max),
+    >>> librosa.display.specshow(librosa.amplitude_to_db(H,
+    ...                                                  ref=np.max),
     ...                          y_axis='log')
     >>> plt.colorbar(format='%+2.0f dB')
     >>> plt.title('Harmonic power spectrogram')
     >>> plt.subplot(3, 1, 3)
-    >>> librosa.display.specshow(librosa.logamplitude(np.abs(P)**2,
-    ...                          ref_power=np.max),
+    >>> librosa.display.specshow(librosa.amplitude_to_db(P,
+    ...                                                  ref=np.max),
     ...                          y_axis='log')
     >>> plt.colorbar(format='%+2.0f dB')
     >>> plt.title('Percussive power spectrogram')
@@ -291,18 +301,17 @@ def hpss(S, kernel_size=31, power=2.0, mask=False, margin=1.0):
 
     >>> mask_H, mask_P = librosa.decompose.hpss(D, mask=True)
     >>> mask_H
-    array([[ 1.,  0., ...,  0.,  0.],
-           [ 1.,  0., ...,  0.,  0.],
+    array([[  1.000e+00,   1.469e-01, ...,   2.648e-03,   2.164e-03],
+           [  1.000e+00,   2.368e-01, ...,   9.413e-03,   7.703e-03],
            ...,
-           [ 0.,  0., ...,  0.,  0.],
-           [ 0.,  0., ...,  0.,  0.]])
+           [  8.869e-01,   5.673e-02, ...,   4.603e-02,   1.247e-05],
+           [  7.068e-01,   2.194e-02, ...,   4.453e-02,   1.205e-05]], dtype=float32)
     >>> mask_P
-    array([[ 0.,  1., ...,  1.,  1.],
-           [ 0.,  1., ...,  1.,  1.],
+    array([[  2.858e-05,   8.531e-01, ...,   9.974e-01,   9.978e-01],
+           [  1.586e-05,   7.632e-01, ...,   9.906e-01,   9.923e-01],
            ...,
-           [ 1.,  1., ...,  1.,  1.],
-           [ 1.,  1., ...,  1.,  1.]])
-
+           [  1.131e-01,   9.433e-01, ...,   9.540e-01,   1.000e+00],
+           [  2.932e-01,   9.781e-01, ...,   9.555e-01,   1.000e+00]], dtype=float32)
 
     Separate into harmonic/percussive/residual components by using a margin > 1.0
 
@@ -340,7 +349,8 @@ def hpss(S, kernel_size=31, power=2.0, mask=False, margin=1.0):
 
     # margin minimum is 1.0
     if margin_harm < 1 or margin_perc < 1:
-        raise ParameterError("Margins must be >= 1.0. A typical range is between 1 and 10.")
+        raise ParameterError("Margins must be >= 1.0. "
+                             "A typical range is between 1 and 10.")
 
     # Compute median filters. Pre-allocation here preserves memory layout.
     harm = np.empty_like(S)
@@ -360,20 +370,12 @@ def hpss(S, kernel_size=31, power=2.0, mask=False, margin=1.0):
                               split_zeros=split_zeros)
 
     if mask:
-        # If the margins are tight, H + P should sum to 1, so we use >=.
-        # Otherwise, comparisons should be strict on both sides.
-        if margin_harm == 1 and margin_perc == 1:
-            compare = np.greater_equal
-        else:
-            compare = np.greater
-
-        return ((mask_harm > mask_perc).astype(float),
-                compare(mask_perc, mask_harm).astype(float))
+        return mask_harm, mask_perc
 
     return ((S * mask_harm) * phase, (S * mask_perc) * phase)
 
 
-@cache
+@cache(level=30)
 def nn_filter(S, rec=None, aggregate=None, axis=-1, **kwargs):
     '''Filtering by nearest-neighbors.
 
@@ -439,6 +441,12 @@ def nn_filter(S, rec=None, aggregate=None, axis=-1, **kwargs):
     hpss
     librosa.segment.recurrence_matrix
 
+
+    Notes
+    -----
+    This function caches at level 30.
+
+
     Examples
     --------
 
